@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-2.1-or-later
 /*
  *
  *  BlueZ - Bluetooth protocol stack for Linux
@@ -5,29 +6,15 @@
  *  Copyright (C) 2012-2014  Intel Corporation. All rights reserved.
  *
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <ctype.h>
-#include <stdarg.h>
 #include <stdbool.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -54,20 +41,30 @@ void *btd_malloc(size_t size)
 	return NULL;
 }
 
+void util_debug_va(util_debug_func_t function, void *user_data,
+				const char *format, va_list va)
+{
+	char str[78];
+
+	if (!function || !format)
+		return;
+
+	vsnprintf(str, sizeof(str), format, va);
+
+	function(str, user_data);
+}
+
 void util_debug(util_debug_func_t function, void *user_data,
 						const char *format, ...)
 {
-	char str[78];
 	va_list ap;
 
 	if (!function || !format)
 		return;
 
 	va_start(ap, format);
-	vsnprintf(str, sizeof(str), format, ap);
+	util_debug_va(function, user_data, format, ap);
 	va_end(ap);
-
-	function(str, user_data);
 }
 
 void util_hexdump(const char dir, const unsigned char *buf, size_t len,
@@ -139,7 +136,7 @@ uint8_t util_get_uid(unsigned int *bitmap, uint8_t max)
 	if (!id || id > max)
 		return 0;
 
-	*bitmap |= 1 << (id - 1);
+	*bitmap |= 1u << (id - 1);
 
 	return id;
 }
@@ -150,7 +147,7 @@ void util_clear_uid(unsigned int *bitmap, uint8_t id)
 	if (!id)
 		return;
 
-	*bitmap &= ~(1 << (id - 1));
+	*bitmap &= ~(1u << (id - 1));
 }
 
 static const struct {
@@ -525,12 +522,14 @@ static const struct {
 	{ 0x2adc, "Mesh Provisioning Data Out"			},
 	{ 0x2add, "Mesh Proxy Data In"				},
 	{ 0x2ade, "Mesh Proxy Data Out"				},
+	{ 0x2b29, "Client Supported Features"			},
+	{ 0x2b2A, "Database Hash"				},
 	/* vendor defined */
 	{ 0xfeff, "GN Netcom"					},
 	{ 0xfefe, "GN ReSound A/S"				},
 	{ 0xfefd, "Gimbal, Inc."				},
 	{ 0xfefc, "Gimbal, Inc."				},
-	{ 0xfefb, "Stollmann E+V GmbH"				},
+	{ 0xfefb, "Telit Wireless Solutions (Formerly Stollmann E+V GmbH)" },
 	{ 0xfefa, "PayPal, Inc."				},
 	{ 0xfef9, "PayPal, Inc."				},
 	{ 0xfef8, "Aplix Corporation"				},
@@ -542,32 +541,32 @@ static const struct {
 	{ 0xfef2, "CSR"						},
 	{ 0xfef1, "CSR"						},
 	{ 0xfef0, "Intel"					},
-	{ 0xfeef, "Polar Electro Oy"				},
-	{ 0xfeee, "Polar Electro Oy"				},
+	{ 0xfeef, "Polar Electro Oy "				},
+	{ 0xfeee, "Polar Electro Oy "				},
 	{ 0xfeed, "Tile, Inc."					},
 	{ 0xfeec, "Tile, Inc."					},
 	{ 0xfeeb, "Swirl Networks, Inc."			},
 	{ 0xfeea, "Swirl Networks, Inc."			},
 	{ 0xfee9, "Quintic Corp."				},
 	{ 0xfee8, "Quintic Corp."				},
-	{ 0xfee7, "Tencent Holdings Limited"			},
-	{ 0xfee6, "Seed Labs, Inc."				},
+	{ 0xfee7, "Tencent Holdings Limited."			},
+	{ 0xfee6, "Silvair, Inc."				},
 	{ 0xfee5, "Nordic Semiconductor ASA"			},
 	{ 0xfee4, "Nordic Semiconductor ASA"			},
 	{ 0xfee3, "Anki, Inc."					},
 	{ 0xfee2, "Anki, Inc."					},
-	{ 0xfee1, "Anhui Huami Information Technology Co."	},
-	{ 0xfee0, "Anhui Huami Information Technology Co."	},
+	{ 0xfee1, "Anhui Huami Information Technology Co., Ltd. " },
+	{ 0xfee0, "Anhui Huami Information Technology Co., Ltd. " },
 	{ 0xfedf, "Design SHIFT"				},
 	{ 0xfede, "Coin, Inc."					},
 	{ 0xfedd, "Jawbone"					},
 	{ 0xfedc, "Jawbone"					},
 	{ 0xfedb, "Perka, Inc."					},
-	{ 0xfeda, "ISSC Technologies Corporation"		},
+	{ 0xfeda, "ISSC Technologies Corp. "			},
 	{ 0xfed9, "Pebble Technology Corporation"		},
 	{ 0xfed8, "Google"					},
-	{ 0xfed7, "Broadcom Corporation"			},
-	{ 0xfed6, "Broadcom Corporation"			},
+	{ 0xfed7, "Broadcom"					},
+	{ 0xfed6, "Broadcom"					},
 	{ 0xfed5, "Plantronics Inc."				},
 	{ 0xfed4, "Apple, Inc."					},
 	{ 0xfed3, "Apple, Inc."					},
@@ -592,25 +591,25 @@ static const struct {
 	{ 0xfec0, "KDDI Corporation"				},
 	{ 0xfebf, "Nod, Inc."					},
 	{ 0xfebe, "Bose Corporation"				},
-	{ 0xfebd, "Clover Network, Inc."			},
-	{ 0xfebc, "Dexcom, Inc."				},
+	{ 0xfebd, "Clover Network, Inc"				},
+	{ 0xfebc, "Dexcom Inc"					},
 	{ 0xfebb, "adafruit industries"				},
 	{ 0xfeba, "Tencent Holdings Limited"			},
 	{ 0xfeb9, "LG Electronics"				},
 	{ 0xfeb8, "Facebook, Inc."				},
 	{ 0xfeb7, "Facebook, Inc."				},
-	{ 0xfeb6, "Vencer Co, Ltd"				},
+	{ 0xfeb6, "Vencer Co., Ltd"				},
 	{ 0xfeb5, "WiSilica Inc."				},
 	{ 0xfeb4, "WiSilica Inc."				},
 	{ 0xfeb3, "Taobao"					},
 	{ 0xfeb2, "Microsoft Corporation"			},
 	{ 0xfeb1, "Electronics Tomorrow Limited"		},
-	{ 0xfeb0, "Nest Labs Inc."				},
-	{ 0xfeaf, "Nest Labs Inc."				},
-	{ 0xfeae, "Nokia Corporation"				},
-	{ 0xfead, "Nokia Corporation"				},
-	{ 0xfeac, "Nokia Corporation"				},
-	{ 0xfeab, "Nokia Corporation"				},
+	{ 0xfeb0, "Nest Labs Inc"				},
+	{ 0xfeaf, "Nest Labs Inc"				},
+	{ 0xfeae, "Nokia"					},
+	{ 0xfead, "Nokia"					},
+	{ 0xfeac, "Nokia"					},
+	{ 0xfeab, "Nokia"					},
 	{ 0xfeaa, "Google"					},
 	{ 0xfea9, "Savant Systems LLC"				},
 	{ 0xfea8, "Savant Systems LLC"				},
@@ -628,13 +627,13 @@ static const struct {
 	{ 0xfe9c, "GSI Laboratories, Inc."			},
 	{ 0xfe9b, "Samsara Networks, Inc"			},
 	{ 0xfe9a, "Estimote"					},
-	{ 0xfe99, "Currant, Inc."				},
-	{ 0xfe98, "Currant, Inc."				},
-	{ 0xfe97, "Tesla Motor Inc."				},
-	{ 0xfe96, "Tesla Motor Inc."				},
+	{ 0xfe99, "Currant Inc"					},
+	{ 0xfe98, "Currant Inc"					},
+	{ 0xfe97, "Tesla Motors Inc."				},
+	{ 0xfe96, "Tesla Motors Inc."				},
 	{ 0xfe95, "Xiaomi Inc."					},
-	{ 0xfe94, "OttoQ Inc."					},
-	{ 0xfe93, "OttoQ Inc."					},
+	{ 0xfe94, "OttoQ In"					},
+	{ 0xfe93, "OttoQ In"					},
 	{ 0xfe92, "Jarden Safety & Security"			},
 	{ 0xfe91, "Shanghai Imilab Technology Co.,Ltd"		},
 	{ 0xfe90, "JUMA"					},
@@ -646,8 +645,8 @@ static const struct {
 	{ 0xfe8a, "Apple, Inc."					},
 	{ 0xfe89, "B&O Play A/S"				},
 	{ 0xfe88, "SALTO SYSTEMS S.L."				},
-	{ 0xfe87, "Qingdao Yeelink Information Technology Co., Ltd. ( 青岛亿联客信息技术有限公司 )"	},
-	{ 0xfe86, "HUAWEI Technologies Co., Ltd. ( 华为技术有限公司 )"					},
+	{ 0xfe87, "Qingdao Yeelink Information Technology Co., Ltd. ( 青岛亿联客信息技术有限公司 )" },
+	{ 0xfe86, "HUAWEI Technologies Co., Ltd. ( 华为技术有限公司 )"	},
 	{ 0xfe85, "RF Digital Corp"				},
 	{ 0xfe84, "RF Digital Corp"				},
 	{ 0xfe83, "Blue Bite"					},
@@ -657,7 +656,7 @@ static const struct {
 	{ 0xfe7f, "Doppler Lab"					},
 	{ 0xfe7e, "Awear Solutions Ltd"				},
 	{ 0xfe7d, "Aterica Health Inc."				},
-	{ 0xfe7c, "Stollmann E+V GmbH"				},
+	{ 0xfe7c, "Telit Wireless Solutions (Formerly Stollmann E+V GmbH)" },
 	{ 0xfe7b, "Orion Labs, Inc."				},
 	{ 0xfe7a, "Bragi GmbH"					},
 	{ 0xfe79, "Zebra Technologies"				},
@@ -666,64 +665,64 @@ static const struct {
 	{ 0xfe76, "TangoMe"					},
 	{ 0xfe75, "TangoMe"					},
 	{ 0xfe74, "unwire"					},
-	{ 0xfe73, "St. Jude Medical, Inc."			},
-	{ 0xfe72, "St. Jude Medical, Inc."			},
+	{ 0xfe73, "Abbott (formerly St. Jude Medical, Inc.)"	},
+	{ 0xfe72, "Abbott (formerly St. Jude Medical, Inc.)"	},
 	{ 0xfe71, "Plume Design Inc"				},
 	{ 0xfe70, "Beijing Jingdong Century Trading Co., Ltd."	},
 	{ 0xfe6f, "LINE Corporation"				},
-	{ 0xfe6e, "The University of Tokyo"			},
-	{ 0xfe6d, "The University of Tokyo"			},
+	{ 0xfe6e, "The University of Tokyo "			},
+	{ 0xfe6d, "The University of Tokyo "			},
 	{ 0xfe6c, "TASER International, Inc."			},
 	{ 0xfe6b, "TASER International, Inc."			},
 	{ 0xfe6a, "Kontakt Micro-Location Sp. z o.o."		},
-	{ 0xfe69, "Qualcomm Life Inc"				},
-	{ 0xfe68, "Qualcomm Life Inc"				},
+	{ 0xfe69, "Capsule Technologies Inc."			},
+	{ 0xfe68, "Capsule Technologies Inc."			},
 	{ 0xfe67, "Lab Sensor Solutions"			},
-	{ 0xfe66, "Intel Corporation"				},
-	{ 0xfe65, "CHIPOLO d.o.o."				},
+	{ 0xfe66, "Intel Corporation "				},
+	{ 0xfe65, "CHIPOLO d.o.o. "				},
 	{ 0xfe64, "Siemens AG"					},
-	{ 0xfe63, "Connected Yard, Inc."			},
-	{ 0xfe62, "Indagem Tech LLC"				},
-	{ 0xfe61, "Logitech International SA"			},
+	{ 0xfe63, "Connected Yard, Inc. "			},
+	{ 0xfe62, "Indagem Tech LLC "				},
+	{ 0xfe61, "Logitech International SA "			},
 	{ 0xfe60, "Lierda Science & Technology Group Co., Ltd."	},
-	{ 0xfe5F, "Eyefi, Inc."					},
-	{ 0xfe5E, "Plastc Corporation"				},
-	{ 0xfe5D, "Grundfos A/S"				},
-	{ 0xfe5C, "million hunters GmbH"			},
-	{ 0xfe5B, "GT-tronics HK Ltd"				},
-	{ 0xfe5A, "Chronologics Corporation"			},
-	{ 0xfe59, "Nordic Semiconductor ASA"			},
-	{ 0xfe58, "Nordic Semiconductor ASA"			},
-	{ 0xfe57, "Dotted Labs"					},
-	{ 0xfe56, "Google Inc."					},
-	{ 0xfe55, "Google Inc."					},
-	{ 0xfe54, "Motiv, Inc."					},
+	{ 0xfe5f, "Eyefi, Inc."					},
+	{ 0xfe5e, "Plastc Corporation "				},
+	{ 0xfe5d, "Grundfos A/S "				},
+	{ 0xfe5c, "million hunters GmbH "			},
+	{ 0xfe5b, "GT-tronics HK Ltd"				},
+	{ 0xfe5a, "Cronologics Corporation"			},
+	{ 0xfe59, "Nordic Semiconductor ASA "			},
+	{ 0xfe58, "Nordic Semiconductor ASA "			},
+	{ 0xfe57, "Dotted Labs "				},
+	{ 0xfe56, "Google Inc. "				},
+	{ 0xfe55, "Google Inc. "				},
+	{ 0xfe54, "Motiv, Inc. "				},
 	{ 0xfe53, "3M"						},
-	{ 0xfe52, "SetPoint Medical"				},
-	{ 0xfe51, "SRAM"					},
+	{ 0xfe52, "SetPoint Medical "				},
+	{ 0xfe51, "SRAM "					},
 	{ 0xfe50, "Google Inc."					},
-	{ 0xfe4F, "Molekule, Inc."				},
-	{ 0xfe4E, "NTT docomo"					},
-	{ 0xfe4D, "Casambi Technologies Oy"			},
-	{ 0xfe4C, "Volkswagen AG"				},
-	{ 0xfe4B, "Koninklijke Philips N.V."			},
-	{ 0xfe4A, "OMRON HEALTHCARE Co., Ltd."			},
+	{ 0xfe4f, "Molekule, Inc."				},
+	{ 0xfe4e, "NTT docomo "					},
+	{ 0xfe4d, "Casambi Technologies Oy"			},
+	{ 0xfe4c, "Volkswagen AG "				},
+	{ 0xfe4b, "Signify Netherlands B.V. (formerly Philips Lighting B.V.)" },
+	{ 0xfe4a, "OMRON HEALTHCARE Co., Ltd."			},
 	{ 0xfe49, "SenionLab AB"				},
-	{ 0xfe48, "General Motors"				},
-	{ 0xfe47, "General Motors"				},
-	{ 0xfe46, "B&O Play A/S"				},
+	{ 0xfe48, "General Motors "				},
+	{ 0xfe47, "General Motors "				},
+	{ 0xfe46, "B&O Play A/S "				},
 	{ 0xfe45, "Snapchat Inc"				},
-	{ 0xfe44, "SK Telecom"					},
+	{ 0xfe44, "SK Telecom "					},
 	{ 0xfe43, "Andreas Stihl AG & Co. KG"			},
-	{ 0xfe42, "Nets A/S"					},
+	{ 0xfe42, "Nets A/S "					},
 	{ 0xfe41, "Inugo Systems Limited"			},
 	{ 0xfe40, "Inugo Systems Limited"			},
-	{ 0xfe3F, "Friday Labs Limited"				},
-	{ 0xfe3E, "BD Medical"					},
-	{ 0xfe3D, "BD Medical"					},
-	{ 0xfe3C, "Alibaba"					},
-	{ 0xfe3B, "Dolby Laboratories"				},
-	{ 0xfe3A, "TTS Tooltechnic Systems AG & Co. KG"		},
+	{ 0xfe3f, "Friday Labs Limited"				},
+	{ 0xfe3e, "BD Medical"					},
+	{ 0xfe3d, "BD Medical"					},
+	{ 0xfe3c, "alibaba"					},
+	{ 0xfe3b, "Dobly Laboratories"				},
+	{ 0xfe3a, "TTS Tooltechnic Systems AG & Co. KG"		},
 	{ 0xfe39, "TTS Tooltechnic Systems AG & Co. KG"		},
 	{ 0xfe38, "Spaceek LTD"					},
 	{ 0xfe37, "Spaceek LTD"					},
@@ -734,26 +733,215 @@ static const struct {
 	{ 0xfe32, "Pro-Mark, Inc."				},
 	{ 0xfe31, "Volkswagen AG"				},
 	{ 0xfe30, "Volkswagen AG"				},
-	{ 0xfe2F, "CRESCO Wireless, Inc"			},
-	{ 0xfe2E, "ERi,Inc."					},
-	{ 0xfe2D, "SMART INNOVATION Co.,Ltd"			},
-	{ 0xfe2C, "Google Inc."					},
-	{ 0xfe2B, "ITT Industries"				},
-	{ 0xfe2A, "DaisyWorks, Inc."				},
+	{ 0xfe2f, "CRESCO Wireless, Inc"			},
+	{ 0xfe2e, "ERi,Inc."					},
+	{ 0xfe2d, "SMART INNOVATION Co.,Ltd"			},
+	{ 0xfe2c, "Google"					},
+	{ 0xfe2b, "ITT Industries"				},
+	{ 0xfe2a, "DaisyWorks, Inc."				},
 	{ 0xfe29, "Gibson Innovations"				},
-	{ 0xfe28, "Ayla Network"				},
-	{ 0xfe27, "Google Inc."					},
-	{ 0xfe26, "Google Inc."					},
-	{ 0xfe25, "Apple, Inc."					},
+	{ 0xfe28, "Ayla Networks"				},
+	{ 0xfe27, "Google"					},
+	{ 0xfe26, "Google"					},
+	{ 0xfe25, "Apple, Inc. "				},
 	{ 0xfe24, "August Home Inc"				},
 	{ 0xfe23, "Zoll Medical Corporation"			},
 	{ 0xfe22, "Zoll Medical Corporation"			},
 	{ 0xfe21, "Bose Corporation"				},
 	{ 0xfe20, "Emerson"					},
-	{ 0xfe1F, "Garmin International, Inc."			},
-	{ 0xfe1E, "Smart Innovations Co., Ltd"			},
-	{ 0xfe1D, "Illuminati Instrument Corporation"		},
-	{ 0xfe1C, "NetMedia, Inc."				},
+	{ 0xfe1f, "Garmin International, Inc."			},
+	{ 0xfe1e, "Smart Innovations Co., Ltd"			},
+	{ 0xfe1d, "Illuminati Instrument Corporation"		},
+	{ 0xfe1c, "NetMedia, Inc."				},
+	{ 0xfe1b, "Tyto Life LLC"				},
+	{ 0xfe1a, "Tyto Life LLC"				},
+	{ 0xfe19, "Google, Inc"					},
+	{ 0xfe18, "Runtime, Inc."				},
+	{ 0xfe17, "Telit Wireless Solutions GmbH"		},
+	{ 0xfe16, "Footmarks, Inc."				},
+	{ 0xfe15, "Amazon.com Services, Inc.."			},
+	{ 0xfe14, "Flextronics International USA Inc."		},
+	{ 0xfe13, "Apple Inc."					},
+	{ 0xfe12, "M-Way Solutions GmbH"			},
+	{ 0xfe11, "GMC-I Messtechnik GmbH"			},
+	{ 0xfe10, "Lapis Semiconductor Co., Ltd."		},
+	{ 0xfe0f, "Signify Netherlands B.V. (formerly Philips Lighting B.V.)" },
+	{ 0xfe0e, "Setec Pty Ltd"				},
+	{ 0xfe0d, "Procter & Gamble"				},
+	{ 0xfe0c, "Procter & Gamble"				},
+	{ 0xfe0b, "ruwido austria gmbh"				},
+	{ 0xfe0a, "ruwido austria gmbh"				},
+	{ 0xfe09, "Pillsy, Inc."				},
+	{ 0xfe08, "Microsoft"					},
+	{ 0xfe07, "Sonos, Inc."					},
+	{ 0xfe06, "Qualcomm Technologies, Inc."			},
+	{ 0xfe05, "CORE Transport Technologies NZ Limited "	},
+	{ 0xfe04, "OpenPath Security Inc"			},
+	{ 0xfe03, "Amazon.com Services, Inc."			},
+	{ 0xfe02, "Robert Bosch GmbH"				},
+	{ 0xfe01, "Duracell U.S. Operations Inc."		},
+	{ 0xfe00, "Amazon.com Services, Inc."			},
+	{ 0xfdff, "OSRAM GmbH"					},
+	{ 0xfdfe, "ADHERIUM(NZ) LIMITED"			},
+	{ 0xfdfd, "RecursiveSoft Inc."				},
+	{ 0xfdfc, "Optrel AG"					},
+	{ 0xfdfb, "Tandem Diabetes Care"			},
+	{ 0xfdfa, "Tandem Diabetes Care"			},
+	{ 0xfdf9, "INIA"					},
+	{ 0xfdf8, "Onvocal"					},
+	{ 0xfdf7, "HP Inc."					},
+	{ 0xfdf6, "AIAIAI ApS"					},
+	{ 0xfdf5, "Milwaukee Electric Tools"			},
+	{ 0xfdf4, "O. E. M. Controls, Inc."			},
+	{ 0xfdf3, "Amersports"					},
+	{ 0xfdf2, "AMICCOM Electronics Corporation"		},
+	{ 0xfdf1, "LAMPLIGHT Co.,Ltd"				},
+	{ 0xfdf0, "Google Inc."					},
+	{ 0xfdef, "ART AND PROGRAM, INC."			},
+	{ 0xfdee, "Huawei Technologies Co., Ltd."		},
+	{ 0xfded, "Pole Star"					},
+	{ 0xfdec, "Mannkind Corporation"			},
+	{ 0xfdeb, "Syntronix Corporation"			},
+	{ 0xfdea, "SeeScan, Inc"				},
+	{ 0xfde9, "Spacesaver Corporation"			},
+	{ 0xfde8, "Robert Bosch GmbH"				},
+	{ 0xfde7, "SECOM Co., LTD"				},
+	{ 0xfde6, "Intelletto Technologies Inc"			},
+	{ 0xfde5, "SMK Corporation "				},
+	{ 0xfde4, "JUUL Labs, Inc."				},
+	{ 0xfde3, "Abbott Diabetes Care"			},
+	{ 0xfde2, "Google Inc."					},
+	{ 0xfde1, "Fortin Electronic Systems "			},
+	{ 0xfde0, "John Deere"					},
+	{ 0xfddf, "Harman International"			},
+	{ 0xfdde, "Noodle Technology Inc. "			},
+	{ 0xfddd, "Arch Systems Inc"				},
+	{ 0xfddc, "4iiii Innovations Inc."			},
+	{ 0xfddb, "Samsung Electronics Co., Ltd. "		},
+	{ 0xfdda, "MHCS"					},
+	{ 0xfdd9, "Jiangsu Teranovo Tech Co., Ltd."		},
+	{ 0xfdd8, "Jiangsu Teranovo Tech Co., Ltd."		},
+	{ 0xfdd7, "Emerson"					},
+	{ 0xfdd6, "Ministry of Supply "				},
+	{ 0xfdd5, "Brompton Bicycle Ltd"			},
+	{ 0xfdd4, "LX Solutions Pty Limited"			},
+	{ 0xfdd3, "FUBA Automotive Electronics GmbH"		},
+	{ 0xfdd2, "Bose Corporation"				},
+	{ 0xfdd1, "Huawei Technologies Co., Ltd "		},
+	{ 0xfdd0, "Huawei Technologies Co., Ltd "		},
+	{ 0xfdcf, "Nalu Medical, Inc"				},
+	{ 0xfdce, "SENNHEISER electronic GmbH & Co. KG"		},
+	{ 0xfdcd, "Qingping Technology (Beijing) Co., Ltd."	},
+	{ 0xfdcc, "Shoof Technologies"				},
+	{ 0xfdcb, "Meggitt SA"					},
+	{ 0xfdca, "Fortin Electronic Systems "			},
+	{ 0xfdc9, "Busch-Jaeger Elektro GmbH"			},
+	{ 0xfdc8, "Hach – Danaher"				},
+	{ 0xfdc7, "Eli Lilly and Company"			},
+	{ 0xfdc6, "Eli Lilly and Company"			},
+	{ 0xfdc5, "Automatic Labs"				},
+	{ 0xfdc4, "Simavita (Aust) Pty Ltd"			},
+	{ 0xfdc3, "Baidu Online Network Technology (Beijing) Co., Ltd" },
+	{ 0xfdc2, "Baidu Online Network Technology (Beijing) Co., Ltd" },
+	{ 0xfdc1, "Hunter Douglas"				},
+	{ 0xfdc0, "Hunter Douglas"				},
+	{ 0xfdbf, "California Things Inc. "			},
+	{ 0xfdbe, "California Things Inc. "			},
+	{ 0xfdbd, "Clover Network, Inc."			},
+	{ 0xfdbc, "Emerson"					},
+	{ 0xfdbb, "Profoto"					},
+	{ 0xfdba, "Comcast Cable Corporation"			},
+	{ 0xfdb9, "Comcast Cable Corporation"			},
+	{ 0xfdb8, "LivaNova USA Inc."				},
+	{ 0xfdb7, "LivaNova USA Inc."				},
+	{ 0xfdb6, "GWA Hygiene GmbH"				},
+	{ 0xfdb5, "ECSG"					},
+	{ 0xfdb4, "HP Inc"					},
+	{ 0xfdb3, "Audiodo AB"					},
+	{ 0xfdb2, "Portable Multimedia Ltd "			},
+	{ 0xfdb1, "Proxy Technologies, Inc."			},
+	{ 0xfdb0, "Proxy Technologies, Inc."			},
+	{ 0xfdaf, "Wiliot LTD"					},
+	{ 0xfdae, "Houwa System Design, k.k."			},
+	{ 0xfdad, "Houwa System Design, k.k."			},
+	{ 0xfdac, "Tentacle Sync GmbH"				},
+	{ 0xfdab, "Xiaomi Inc."					},
+	{ 0xfdaa, "Xiaomi Inc."					},
+	{ 0xfda9, "Rhombus Systems, Inc."			},
+	{ 0xfda8, "PSA Peugeot Citroën"				},
+	{ 0xfda7, "WWZN Information Technology Company Limited"	},
+	{ 0xfda6, "WWZN Information Technology Company Limited"	},
+	{ 0xfda5, "Neurostim OAB, Inc."				},
+	{ 0xfda4, "Inseego Corp."				},
+	{ 0xfda3, "Inseego Corp."				},
+	{ 0xfda2, "Groove X, Inc"				},
+	{ 0xfda1, "Groove X, Inc"				},
+	{ 0xfda0, "Secugen Corporation"				},
+	{ 0xfd9f, "VitalTech Affiliates LLC"			},
+	{ 0xfd9e, "The Coca-Cola Company"			},
+	{ 0xfd9d, "Gastec Corporation"				},
+	{ 0xfd9c, "Huawei Technologies Co., Ltd."		},
+	{ 0xfd9b, "Huawei Technologies Co., Ltd."		},
+	{ 0xfd9a, "Huawei Technologies Co., Ltd."		},
+	{ 0xfd99, "ABB Oy"					},
+	{ 0xfd98, "Disney Worldwide Services, Inc."		},
+	{ 0xfd97, "June Life, Inc."				},
+	{ 0xfd96, "Google LLC"					},
+	{ 0xfd95, "Rigado"					},
+	{ 0xfd94, "Hewlett Packard Enterprise"			},
+	{ 0xfd93, "Bayerische Motoren Werke AG"			},
+	{ 0xfd92, "Qualcomm Technologies International, Ltd. (QTIL)" },
+	{ 0xfd91, "Groove X, Inc."				},
+	{ 0xfd90, "Guangzhou SuperSound Information Technology Co.,Ltd" },
+	{ 0xfd8f, "Matrix ComSec Pvt. Ltd."			},
+	{ 0xfd8e, "Motorola Solutions"				},
+	{ 0xfd8d, "quip NYC Inc."				},
+	{ 0xfd8c, "Google LLC"					},
+	{ 0xfd8b, "Jigowatts Inc."				},
+	{ 0xfd8a, "Signify Netherlands B.V."			},
+	{ 0xfd89, "Urbanminded LTD"				},
+	{ 0xfd88, "Urbanminded LTD"				},
+	{ 0xfd87, "Google LLC"					},
+	{ 0xfd86, "Abbott"					},
+	{ 0xfd85, "Husqvarna AB"				},
+	{ 0xfd84, "Tile, Inc."					},
+	{ 0xfd83, "iNFORM Technology GmbH"			},
+	{ 0xfd82, "Sony Corporation"				},
+	{ 0xfd81, "CANDY HOUSE, Inc."				},
+	{ 0xfd80, "Phindex Technologies, Inc"			},
+	{ 0xfd7f, "Husqvarna AB"				},
+	{ 0xfd7e, "Samsung Electronics Co., Ltd."		},
+	{ 0xfd7d, "Center for Advanced Research Wernher Von Braun" },
+	{ 0xfd7c, "Toshiba Information Systems(Japan) Corporation" },
+	{ 0xfd7b, "WYZE LABS, INC."				},
+	{ 0xfd7a, "Withings"					},
+	{ 0xfd79, "Withings"					},
+	{ 0xfd78, "Withings"					},
+	{ 0xfd77, "Withings"					},
+	{ 0xfd76, "Insulet Corporation"				},
+	{ 0xfd75, "Insulet Corporation"				},
+	{ 0xfd74, "BRControls Products BV"			},
+	{ 0xfd73, "BRControls Products BV"			},
+	{ 0xfd72, "Logitech International SA"			},
+	{ 0xfd71, "GN Hearing A/S"				},
+	{ 0xfd70, "GuangDong Oppo Mobile Telecommunications Corp., Ltd." },
+	{ 0xfd6f, "Apple, Inc."					},
+	{ 0xfd6e, "Polidea sp. z o.o."				},
+	{ 0xfd6d, "Sigma Elektro GmbH"				},
+	{ 0xfd6c, "Samsung Electronics Co., Ltd."		},
+	{ 0xfd6b, " rapitag GmbH"				},
+	{ 0xfd6a, "Emerson"					},
+	{ 0xfd69, "Samsung Electronics Co., Ltd."		},
+	{ 0xfd68, "Ubique Innovation AG"			},
+	{ 0xfd67, "Montblanc Simplo GmbH"			},
+	{ 0xfd66, "Zebra Technologies Corporation"		},
+	{ 0xfd65, "Razer Inc."					},
+	{ 0xfd64, "INRIA"					},
+	{ 0xfd63, "Fitbit, Inc."				},
+	{ 0xfd62, "Fitbit, Inc."				},
+	{ 0xfd61, "Arendi AG"					},
+	{ 0xfd60, "Sercomm Corporation"				},
+	{ 0xfd5f, "Oculus VR, LLC"				},
 	/* SDO defined */
 	{ 0xfffc, "AirFuel Alliance"				},
 	{ 0xfffe, "Alliance for Wireless Power (A4WP)"		},
